@@ -125,7 +125,7 @@ case "${LSBNAME}" in
     replace_uring_with_aio
     disable_libfmt
     ;&
-  impish|jammy)
+  impish|jammy|kinetic)
     # mariadb-plugin-rocksdb s390x not supported by us (yet)
     # ubuntu doesn't support mips64el yet, so keep this just
     # in case something changes.
