@@ -55,6 +55,7 @@ Street, Fifth Floor, Boston, MA 02110-1335 USA
 #include "xtrabackup.h"
 #include "common.h"
 #include "backup_copy.h"
+#include "backup_debug.h"
 #include "backup_mysql.h"
 #include <btr0btr.h>
 
@@ -1444,6 +1445,13 @@ bool backup_start(CorruptedPages &corrupted_pages)
 
 	msg("Waiting for log copy thread to read lsn %llu", (ulonglong)server_lsn_after_lock);
 	backup_wait_for_lsn(server_lsn_after_lock);
+	DBUG_EXECUTE_FOR_KEY("sleep_after_waiting_for_lsn", {},
+		{
+			ulong milliseconds = strtoul(dbug_val, NULL, 10);
+			msg("sleep_after_waiting_for_lsn");
+			my_sleep(milliseconds*1000UL);
+		});
+
 	backup_fix_ddl(corrupted_pages);
 
 	// There is no need to stop slave thread before coping non-Innodb data when
@@ -1475,6 +1483,7 @@ bool backup_start(CorruptedPages &corrupted_pages)
 		if (!write_galera_info(mysql_connection)) {
 			return(false);
 		}
+                // copied from xtrabackup. what is it needed for here?
 		write_current_binlog_file(mysql_connection);
 	}
 
@@ -1484,7 +1493,7 @@ bool backup_start(CorruptedPages &corrupted_pages)
 		write_binlog_info(mysql_connection);
 	}
 
-	if (have_flush_engine_logs && !opt_no_lock) {
+	if (!opt_no_lock) {
 		msg("Executing FLUSH NO_WRITE_TO_BINLOG ENGINE LOGS...");
 		xb_mysql_query(mysql_connection,
 			"FLUSH NO_WRITE_TO_BINLOG ENGINE LOGS", false);
@@ -1515,16 +1524,15 @@ void backup_release()
 	}
 }
 
+static const char *default_buffer_pool_file = "ib_buffer_pool";
+
 /** Finish after backup_start() and backup_release() */
 bool backup_finish()
 {
 	/* Copy buffer pool dump or LRU dump */
-	if (!opt_rsync) {
+	if (!opt_rsync && opt_galera_info) {
 		if (buffer_pool_filename && file_exists(buffer_pool_filename)) {
-			const char *dst_name;
-
-			dst_name = trim_dotslash(buffer_pool_filename);
-			copy_file(ds_data, buffer_pool_filename, dst_name, 0);
+			copy_file(ds_data, buffer_pool_filename, default_buffer_pool_file, 0);
 		}
 		if (file_exists("ib_lru_dump")) {
 			copy_file(ds_data, "ib_lru_dump", "ib_lru_dump", 0);
@@ -1574,6 +1582,7 @@ ibx_copy_incremental_over_full()
 	char path[FN_REFLEN];
 	int i;
 
+        DBUG_ASSERT(!opt_galera_info);
 	datadir_node_init(&node);
 
 	/* If we were applying an incremental change set, we need to make
@@ -1610,22 +1619,6 @@ ibx_copy_incremental_over_full()
 		if (!(ret = backup_files_from_datadir(xtrabackup_incremental_dir)))
 			goto cleanup;
 
-		/* copy buffer pool dump */
-		if (innobase_buffer_pool_filename) {
-			const char *src_name;
-
-			src_name = trim_dotslash(innobase_buffer_pool_filename);
-
-			snprintf(path, sizeof(path), "%s/%s",
-				xtrabackup_incremental_dir,
-				src_name);
-
-			if (file_exists(path)) {
-				copy_file(ds_data, path,
-					  innobase_buffer_pool_filename, 0);
-			}
-		}
-
 		/* copy supplementary files */
 
 		for (i = 0; sup_files[i]; i++) {
@@ -1915,6 +1908,11 @@ copy_back()
 			continue;
 		}
 
+                /* skip buffer pool dump */
+                if (!strcmp(filename, default_buffer_pool_file)) {
+                        continue;
+                }
+
 		/* skip innodb data files */
 		for (Tablespace::const_iterator iter(srv_sys_space.begin()),
 		       end(srv_sys_space.end()); iter != end; ++iter) {
@@ -1933,24 +1931,11 @@ copy_back()
 
 	/* copy buffer pool dump */
 
-	if (innobase_buffer_pool_filename) {
-		const char *src_name;
-		char path[FN_REFLEN];
-
-		src_name = trim_dotslash(innobase_buffer_pool_filename);
-
-		snprintf(path, sizeof(path), "%s/%s",
-			mysql_data_home,
-			src_name);
-
-		/* could be already copied with other files
-		from data directory */
-		if (file_exists(src_name) &&
-			!file_exists(innobase_buffer_pool_filename)) {
-			copy_or_move_file(src_name,
-					  innobase_buffer_pool_filename,
-					  mysql_data_home, 0);
-		}
+        if (file_exists(default_buffer_pool_file) &&
+            innobase_buffer_pool_filename) {
+		copy_or_move_file(default_buffer_pool_file,
+				  innobase_buffer_pool_filename,
+				  mysql_data_home, 0);
 	}
 
 	rocksdb_copy_back();
