@@ -62,16 +62,13 @@ Street, Fifth Floor, Boston, MA 02110-1335 USA
 char *tool_name;
 char tool_args[2048];
 
-/* mysql flavor and version */
-mysql_flavor_t server_flavor = FLAVOR_UNKNOWN;
-unsigned long mysql_server_version = 0;
+ulong mysql_server_version;
 
 /* server capabilities */
 bool have_changed_page_bitmaps = false;
 bool have_backup_locks = false;
 bool have_lock_wait_timeout = false;
 bool have_galera_enabled = false;
-bool have_flush_engine_logs = false;
 bool have_multi_threaded_slave = false;
 bool have_gtid_slave = false;
 
@@ -84,6 +81,7 @@ static mysql_cond_t kill_query_thread_stop;
 bool sql_thread_started = false;
 char *mysql_slave_position = NULL;
 char *mysql_binlog_position = NULL;
+char *buffer_pool_filename = NULL;
 
 /* History on server */
 time_t history_start_time;
@@ -134,7 +132,7 @@ xb_mysql_connect()
 	       opt_socket ? opt_socket : "not set");
 
 #ifdef HAVE_OPENSSL
-	if (opt_use_ssl)
+	if (opt_use_ssl && opt_protocol <= MYSQL_PROTOCOL_SOCKET)
 	{
 		mysql_ssl_set(connection, opt_ssl_key, opt_ssl_cert,
 			      opt_ssl_ca, opt_ssl_capath,
@@ -297,59 +295,22 @@ read_mysql_one_value(MYSQL *mysql, const char *query)
 
 static
 bool
-check_server_version(unsigned long version_number,
-		     const char *version_string,
-		     const char *version_comment,
-		     const char *innodb_version)
+check_server_version(ulong version_number, const char *version_string)
 {
-	bool version_supported = false;
-	bool mysql51 = false;
+  if (strstr(version_string, "MariaDB") && version_number >= 100800)
+    return true;
 
-	mysql_server_version = version_number;
-
-	server_flavor = FLAVOR_UNKNOWN;
-	if (strstr(version_comment, "Percona") != NULL) {
-		server_flavor = FLAVOR_PERCONA_SERVER;
-	} else if (strstr(version_comment, "MariaDB") != NULL ||
-		   strstr(version_string, "MariaDB") != NULL) {
-		server_flavor = FLAVOR_MARIADB;
-	} else if (strstr(version_comment, "MySQL") != NULL) {
-		server_flavor = FLAVOR_MYSQL;
-	}
-
-	mysql51 = version_number > 50100 && version_number < 50500;
-	version_supported = version_supported
-		|| (mysql51 && innodb_version != NULL);
-	version_supported = version_supported
-		|| (version_number > 50500 && version_number < 50700);
-	version_supported = version_supported
-		|| ((version_number > 100000)
-		    && server_flavor == FLAVOR_MARIADB);
-
-	if (mysql51 && innodb_version == NULL) {
-		msg("Error: Built-in InnoDB in MySQL 5.1 is not "
-		    "supported in this release. You can either use "
-		    "Percona XtraBackup 2.0, or upgrade to InnoDB "
-		    "plugin.");
-	} else if (!version_supported) {
-		msg("Error: Unsupported server version: '%s'. Please "
-		    "report a bug at "
-		    "https://bugs.launchpad.net/percona-xtrabackup",
-		    version_string);
-	}
-
-	return(version_supported);
+  msg("Error: Unsupported server version: '%s'.", version_string);
+  return false;
 }
 
 /*********************************************************************//**
-Receive options important for XtraBackup from MySQL server.
+Receive options important for XtraBackup from server.
 @return	true on success. */
 bool get_mysql_vars(MYSQL *connection)
 {
   char *gtid_mode_var= NULL;
   char *version_var= NULL;
-  char *version_comment_var= NULL;
-  char *innodb_version_var= NULL;
   char *have_backup_locks_var= NULL;
   char *log_bin_var= NULL;
   char *lock_wait_timeout_var= NULL;
@@ -369,7 +330,7 @@ bool get_mysql_vars(MYSQL *connection)
   char *page_zip_level_var= NULL;
   char *ignore_db_dirs= NULL;
   char *endptr;
-  unsigned long server_version= mysql_get_server_version(connection);
+  ulong server_version= mysql_get_server_version(connection);
 
   bool ret= true;
 
@@ -379,8 +340,6 @@ bool get_mysql_vars(MYSQL *connection)
       {"lock_wait_timeout", &lock_wait_timeout_var},
       {"gtid_mode", &gtid_mode_var},
       {"version", &version_var},
-      {"version_comment", &version_comment_var},
-      {"innodb_version", &innodb_version_var},
       {"wsrep_on", &wsrep_on_var},
       {"slave_parallel_workers", &slave_parallel_workers_var},
       {"gtid_slave_pos", &gtid_slave_pos_var},
@@ -423,18 +382,13 @@ bool get_mysql_vars(MYSQL *connection)
     have_galera_enabled= true;
   }
 
-  /* Check server version compatibility and detect server flavor */
-
-  if (!(ret= check_server_version(server_version, version_var,
-                                  version_comment_var, innodb_version_var)))
+  /* Check server version compatibility */
+  if (!(ret= check_server_version(server_version, version_var)))
   {
     goto out;
   }
 
-  if (server_version > 50500)
-  {
-    have_flush_engine_logs= true;
-  }
+  mysql_server_version= server_version;
 
   if (slave_parallel_workers_var != NULL &&
       atoi(slave_parallel_workers_var) > 0)
@@ -565,16 +519,6 @@ detect_mysql_capabilities_for_backup()
 
 		have_changed_page_bitmaps = (atoi(innodb_changed_pages) == 1);
 
-		/* INNODB_CHANGED_PAGES are listed in
-		INFORMATION_SCHEMA.PLUGINS in MariaDB, but
-		FLUSH NO_WRITE_TO_BINLOG CHANGED_PAGE_BITMAPS
-		is not supported for versions below 10.1.6
-		(see MDEV-7472) */
-		if (server_flavor == FLAVOR_MARIADB &&
-		    mysql_server_version < 100106) {
-			have_changed_page_bitmaps = false;
-		}
-
 		free_mysql_variables(vars);
 	}
 
@@ -610,7 +554,7 @@ select_incremental_lsn_from_history(lsn_t *incremental_lsn)
 				(unsigned long)strlen(opt_incremental_history_name));
 		snprintf(query, sizeof(query),
 			"SELECT innodb_to_lsn "
-			"FROM PERCONA_SCHEMA.xtrabackup_history "
+			"FROM " XB_HISTORY_TABLE " "
 			"WHERE name = '%s' "
 			"AND innodb_to_lsn IS NOT NULL "
 			"ORDER BY innodb_to_lsn DESC LIMIT 1",
@@ -623,7 +567,7 @@ select_incremental_lsn_from_history(lsn_t *incremental_lsn)
 				(unsigned long)strlen(opt_incremental_history_uuid));
 		snprintf(query, sizeof(query),
 			"SELECT innodb_to_lsn "
-			"FROM PERCONA_SCHEMA.xtrabackup_history "
+			"FROM " XB_HISTORY_TABLE " "
 			"WHERE uuid = '%s' "
 			"AND innodb_to_lsn IS NOT NULL "
 			"ORDER BY innodb_to_lsn DESC LIMIT 1",
@@ -1381,21 +1325,8 @@ bool
 write_slave_info(MYSQL *connection)
 {
   String sql, comment;
-  bool show_all_slaves_status= false;
 
-  switch (server_flavor)
-  {
-  case FLAVOR_MARIADB:
-    show_all_slaves_status= mysql_server_version >= 100000;
-    break;
-  case FLAVOR_UNKNOWN:
-  case FLAVOR_MYSQL:
-  case FLAVOR_PERCONA_SERVER:
-    break;
-  }
-
-  if (Show_slave_status::get_slave_info(connection, show_all_slaves_status,
-                                        &sql, &comment))
+  if (Show_slave_status::get_slave_info(connection, true, &sql, &comment))
     return false; // Error
 
   if (!sql.length())
@@ -1459,29 +1390,27 @@ cleanup:
 }
 
 
-static
-bool
-write_binlog_info(MYSQL *connection, char *log_bin_dir,
-		  MYSQL_RES *mysql_result, my_ulonglong n_rows,
-		  my_ulonglong start);
-
 /*********************************************************************//**
 Flush and copy the current binary log file into the backup,
 if GTID is enabled */
 bool
-write_current_binlog_file(MYSQL *connection, bool write_binlogs)
+write_current_binlog_file(MYSQL *connection)
 {
-	char *log_bin = NULL;
-	char *filename = NULL;
-	char *position = NULL;
 	char *executed_gtid_set = NULL;
 	char *gtid_binlog_state = NULL;
+	char *log_bin_file = NULL;
 	char *log_bin_dir = NULL;
 	bool gtid_exists;
 	bool result = true;
+	char filepath[FN_REFLEN];
 
-	mysql_variable log_bin_var[] = {
-		{"@@GLOBAL.log_bin", &log_bin},
+	mysql_variable status[] = {
+		{"Executed_Gtid_Set", &executed_gtid_set},
+		{NULL, NULL}
+	};
+
+	mysql_variable status_after_flush[] = {
+		{"File", &log_bin_file},
 		{NULL, NULL}
 	};
 
@@ -1491,36 +1420,21 @@ write_current_binlog_file(MYSQL *connection, bool write_binlogs)
 		{NULL, NULL}
 	};
 
-	mysql_variable status[] = {
-		{"File", &filename},
-		{"Position", &position},
-		{"Executed_Gtid_Set", &executed_gtid_set},
-		{NULL, NULL}
-	};
-
-	read_mysql_variables(connection, "SELECT @@GLOBAL.log_bin", log_bin_var, false);
-
-	/* Do not create xtrabackup_binlog_info if binary log is disabled: */
-	if (strncmp(log_bin, "1", 2) != 0) {
-		goto binlog_disabled;
-	}
-
-	lock_binlog_maybe(connection);
-
 	read_mysql_variables(connection, "SHOW MASTER STATUS", status, false);
-
-	/* Do not create xtrabackup_binlog_info if replication
-	has not started yet: */
-	if (filename == NULL || position == NULL) {
-		goto no_replication;
-	}
-
 	read_mysql_variables(connection, "SHOW VARIABLES", vars, true);
 
 	gtid_exists = (executed_gtid_set && *executed_gtid_set)
 			|| (gtid_binlog_state && *gtid_binlog_state);
 
-	if (write_binlogs || gtid_exists) {
+	if (gtid_exists) {
+		size_t log_bin_dir_length;
+
+		lock_binlog_maybe(connection);
+
+		xb_mysql_query(connection, "FLUSH BINARY LOGS", false);
+
+		read_mysql_variables(connection, "SHOW MASTER STATUS",
+			status_after_flush, false);
 
 		if (opt_log_bin != NULL && strchr(opt_log_bin, FN_LIBCHAR)) {
 			/* If log_bin is set, it has priority */
@@ -1530,88 +1444,33 @@ write_current_binlog_file(MYSQL *connection, bool write_binlogs)
 			log_bin_dir = strdup(opt_log_bin);
 		} else if (log_bin_dir == NULL) {
 			/* Default location is MySQL datadir */
-			log_bin_dir = static_cast<char*>(malloc(3));
-			ut_a(log_bin_dir);
-			log_bin_dir[0] = '.';
-			log_bin_dir[1] = FN_LIBCHAR;
-			log_bin_dir[2] = 0;
+			log_bin_dir = strdup("./");
 		}
 
-		size_t log_bin_dir_length;
-
 		dirname_part(log_bin_dir, log_bin_dir, &log_bin_dir_length);
 
 		/* strip final slash if it is not the only path component */
-		while (IS_TRAILING_SLASH(log_bin_dir, log_bin_dir_length)) {
-			log_bin_dir_length--;
+		if (log_bin_dir_length > 1 &&
+		    log_bin_dir[log_bin_dir_length - 1] == FN_LIBCHAR) {
+			log_bin_dir[log_bin_dir_length - 1] = 0;
 		}
-		log_bin_dir[log_bin_dir_length] = 0;
 
-		if (log_bin_dir == NULL) {
-			msg("Failed to locate binary log files");
+		if (log_bin_dir == NULL || log_bin_file == NULL) {
+			msg("Failed to get master binlog coordinates from "
+				"SHOW MASTER STATUS");
 			result = false;
 			goto cleanup;
 		}
 
-		uint max_binlogs;
-		max_binlogs = opt_max_binlogs;
-		if (max_binlogs == 0) {
-			if (gtid_exists) {
-				max_binlogs = 1;
-			} else {
-				goto cleanup;
-			}
-		}
-
-		xb_mysql_query(connection, "FLUSH BINARY LOGS", false);
-
-		MYSQL_RES *mysql_result;
-
-		mysql_result = xb_mysql_query(connection, "SHOW BINARY LOGS", true);
-
-		ut_ad(mysql_num_fields(mysql_result) >= 2);
-
-		my_ulonglong n_rows;
-		my_ulonglong start;
-
-		n_rows = mysql_num_rows(mysql_result);
-
-		start = 0;
-		if (max_binlogs < n_rows) {
-			start = n_rows - max_binlogs;
-		}
-		if (start) {
-			mysql_data_seek(mysql_result, start);
-		}
-
-		MYSQL_ROW row;
-		while ((row = mysql_fetch_row(mysql_result))) {
-			const char *binlog_name = row[0];
-			char filepath[FN_REFLEN];
-			snprintf(filepath, sizeof(filepath), "%s%c%s",
-				 log_bin_dir, FN_LIBCHAR, binlog_name);
-			if (file_exists(filepath)) {
-				result = copy_file(ds_data, filepath, binlog_name, 0);
-				if (!result) break;
-			}
-		}
-
-		if (result) {
-			write_binlog_info(connection, log_bin_dir,
-					  mysql_result, n_rows, start);
-		}
-
-		mysql_free_result(mysql_result);
+		snprintf(filepath, sizeof(filepath), "%s%c%s",
+			 log_bin_dir, FN_LIBCHAR, log_bin_file);
+		result = copy_file(ds_data, filepath, log_bin_file, 0);
 	}
 
 cleanup:
-	free_mysql_variables(vars);
-
-no_replication:
+	free_mysql_variables(status_after_flush);
 	free_mysql_variables(status);
-
-binlog_disabled:
-	free_mysql_variables(log_bin_var);
+	free_mysql_variables(vars);
 
 	return(result);
 }
@@ -1620,11 +1479,8 @@ binlog_disabled:
 /*********************************************************************//**
 Retrieves MySQL binlog position and
 saves it in a file. It also prints it to stdout. */
-static
 bool
-write_binlog_info(MYSQL *connection, char *log_bin_dir,
-		  MYSQL_RES *mysql_result, my_ulonglong n_rows,
-		  my_ulonglong start)
+write_binlog_info(MYSQL *connection)
 {
 	char *filename = NULL;
 	char *position = NULL;
@@ -1632,13 +1488,9 @@ write_binlog_info(MYSQL *connection, char *log_bin_dir,
 	char *gtid_current_pos = NULL;
 	char *gtid_executed = NULL;
 	char *gtid = NULL;
-	char *buffer;
-	char *buf;
-	size_t total;
-	bool result = true;
+	bool result;
 	bool mysql_gtid;
 	bool mariadb_gtid;
-	bool with_gtid;
 
 	mysql_variable status[] = {
 		{"File", &filename},
@@ -1656,106 +1508,39 @@ write_binlog_info(MYSQL *connection, char *log_bin_dir,
 	read_mysql_variables(connection, "SHOW MASTER STATUS", status, false);
 	read_mysql_variables(connection, "SHOW VARIABLES", vars, true);
 
-	mysql_gtid = gtid_mode && (strcmp(gtid_mode, "ON") == 0);
-	mariadb_gtid = gtid_current_pos && *gtid_current_pos;
+	if (filename == NULL || position == NULL) {
+		/* Do not create xtrabackup_binlog_info if binary
+		log is disabled */
+		result = true;
+		goto cleanup;
+	}
 
-	gtid = (gtid_executed && *gtid_executed) ? gtid_executed : gtid_current_pos;
+	mysql_gtid = ((gtid_mode != NULL) && (strcmp(gtid_mode, "ON") == 0));
+	mariadb_gtid = (gtid_current_pos != NULL);
 
-	with_gtid = mariadb_gtid || mysql_gtid;
-	if (with_gtid) {
+	gtid = (gtid_executed != NULL ? gtid_executed : gtid_current_pos);
+
+	if (mariadb_gtid || mysql_gtid) {
 		ut_a(asprintf(&mysql_binlog_position,
 			"filename '%s', position '%s', "
 			"GTID of the last change '%s'",
 			filename, position, gtid) != -1);
+		result = backup_file_printf(XTRABACKUP_BINLOG_INFO,
+					    "%s\t%s\t%s\n", filename, position,
+					    gtid);
 	} else {
 		ut_a(asprintf(&mysql_binlog_position,
 			"filename '%s', position '%s'",
 			filename, position) != -1);
+		result = backup_file_printf(XTRABACKUP_BINLOG_INFO,
+					    "%s\t%s\n", filename, position);
 	}
 
-	mysql_data_seek(mysql_result, start);
-
-	MYSQL_ROW row;
-	my_ulonglong current;
-
-	total = 1;
-	current = start;
-	while ((row = mysql_fetch_row(mysql_result))) {
-		const char *binlog_name = row[0];
-		/* The position in the current binlog is taken from
-		the global variable, but for the previous ones it is
-		determined by their length: */
-		const char *binlog_pos =
-			++current == n_rows ? position : row[1];
-		total += strlen(binlog_name) + strlen(binlog_pos) + 2;
-		if (with_gtid && current != n_rows) {
-			/* Add the "\t[]" length to the buffer size: */
-			total += 3;
-		}
-	}
-	/* For the last of the binray log files, also add
-	the length of the GTID (+ one character for '\t'): */
-	if (with_gtid) {
-		total += strlen(gtid) + 1;
-	}
-
-	buffer = static_cast<char*>(malloc(total));
-	if (!buffer) {
-		msg("Failed to allocate memory for temporary buffer");
-		result = false;
-		goto cleanup;
-	}
-
-	mysql_data_seek(mysql_result, start);
-
-	buf = buffer;
-	current = start;
-	while ((row = mysql_fetch_row(mysql_result))) {
-		const char *binlog_name = row[0];
-		char filepath[FN_REFLEN];
-		snprintf(filepath, sizeof(filepath), "%s%c%s",
-			 log_bin_dir, FN_LIBCHAR, binlog_name);
-		current++;
-		if (file_exists(filepath)) {
-			/* The position in the current binlog is taken from
-			the global variable, but for the previous ones it is
-			determined by their length: */
-			char *binlog_pos =
-				current == n_rows ? position : row[1];
-			int bytes;
-			if (with_gtid) {
-				bytes = snprintf(buf, total, "%s\t%s\t%s\n",
-						 binlog_name, binlog_pos,
-						 current == n_rows ? gtid : "[]");
-			} else {
-				bytes = snprintf(buf, total, "%s\t%s\n",
-						 binlog_name, binlog_pos);
-			}
-			if (bytes <= 0) {
-				goto buffer_overflow;
-			}
-			buf += bytes;
-			total -= bytes;
-		}
-	}
-
-	if (buf != buffer) {
-		result = backup_file_printf(XTRABACKUP_BINLOG_INFO, "%s", buffer);
-	}
-
-cleanup2:
-	free(buffer);
-
 cleanup:
-	free_mysql_variables(vars);
 	free_mysql_variables(status);
+	free_mysql_variables(vars);
 
 	return(result);
-
-buffer_overflow:
-	msg("Internal error: buffer overflow in the write_binlog_info()");
-	result = false;
-	goto cleanup2;
 }
 
 struct escape_and_quote
@@ -1783,7 +1568,7 @@ operator<<(std::ostream& s, const escape_and_quote& eq)
 
 /*********************************************************************//**
 Writes xtrabackup_info file and if backup_history is enable creates
-PERCONA_SCHEMA.xtrabackup_history and writes a new history record to the
+mysql.mariabackup_history and writes a new history record to the
 table containing all the history info particular to the just completed
 backup. */
 bool
@@ -1883,9 +1668,7 @@ write_xtrabackup_info(MYSQL *connection, const char * filename, bool history,
 	}
 
 	xb_mysql_query(connection,
-		"CREATE DATABASE IF NOT EXISTS PERCONA_SCHEMA", false);
-	xb_mysql_query(connection,
-		"CREATE TABLE IF NOT EXISTS PERCONA_SCHEMA.xtrabackup_history("
+		"CREATE TABLE IF NOT EXISTS " XB_HISTORY_TABLE "("
 		"uuid VARCHAR(40) NOT NULL PRIMARY KEY,"
 		"name VARCHAR(255) DEFAULT NULL,"
 		"tool_name VARCHAR(255) DEFAULT NULL,"
@@ -1908,7 +1691,7 @@ write_xtrabackup_info(MYSQL *connection, const char * filename, bool history,
 
 #define ESCAPE_BOOL(expr) ((expr)?"'Y'":"'N'")
 
-	oss << "insert into PERCONA_SCHEMA.xtrabackup_history("
+	oss << "insert into " XB_HISTORY_TABLE "("
 		<< "uuid, name, tool_name, tool_command, tool_version,"
 		<< "ibbackup_version, server_version, start_time, end_time,"
 		<< "lock_time, binlog_pos, innodb_from_lsn, innodb_to_lsn,"
@@ -1988,8 +1771,8 @@ static std::string make_local_paths(const char *data_file_path)
 bool write_backup_config_file()
 {
 	int rc= backup_file_printf("backup-my.cnf",
-		"# This MySQL options file was generated by innobackupex.\n\n"
-		"# The MySQL server\n"
+		"# This options file was generated by innobackupex.\n\n"
+		"# The server\n"
 		"[mysqld]\n"
 		"innodb_checksum_algorithm=%s\n"
 		"innodb_data_file_path=%s\n"
@@ -2074,13 +1857,14 @@ flush_changed_page_bitmaps()
 
 
 /*********************************************************************//**
-Deallocate memory, disconnect from MySQL server, etc.
+Deallocate memory, disconnect from server, etc.
 @return	true on success. */
 void
 backup_cleanup()
 {
 	free(mysql_slave_position);
 	free(mysql_binlog_position);
+	free(buffer_pool_filename);
 
 	if (mysql_connection) {
 		mysql_close(mysql_connection);
