@@ -1,5 +1,5 @@
 /* Copyright (c) 2000, 2010, Oracle and/or its affiliates. All rights reserved.
-   Copyright (c) 2018, 2020, MariaDB
+   Copyright (c) 2018, 2021, MariaDB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -360,7 +360,7 @@ void field_unpack(String *to, Field *field, const uchar *rec, uint max_length,
   {
     if (field->is_null())
     {
-      to->append(STRING_WITH_LEN("NULL"));
+      to->append(NULL_clex_str);
       DBUG_VOID_RETURN;
     }
     CHARSET_INFO *cs= field->charset();
@@ -393,7 +393,7 @@ void field_unpack(String *to, Field *field, const uchar *rec, uint max_length,
     if (max_length < field->pack_length())
       tmp.length(MY_MIN(tmp.length(),max_length));
     ErrConvString err(&tmp);
-    to->append(err.ptr());
+    to->append(err.lex_cstring());
   }
   else
     to->append(STRING_WITH_LEN("???"));
@@ -434,7 +434,7 @@ void key_unpack(String *to, TABLE *table, KEY *key)
     {
       if (table->record[0][key_part->null_offset] & key_part->null_bit)
       {
-	to->append(STRING_WITH_LEN("NULL"));
+	to->append(NULL_clex_str);
 	continue;
       }
     }
@@ -495,6 +495,7 @@ int key_cmp(KEY_PART_INFO *key_part, const uchar *key, uint key_length)
   {
     int cmp;
     store_length= key_part->store_length;
+    int sort_order = (key_part->key_part_flag & HA_REVERSE_SORT) ? -1 : 1;
     if (key_part->null_bit)
     {
       /* This key part allows null values; NULL is lower than everything */
@@ -503,19 +504,19 @@ int key_cmp(KEY_PART_INFO *key_part, const uchar *key, uint key_length)
       {
 	/* the range is expecting a null value */
 	if (!field_is_null)
-	  return 1;                             // Found key is > range
+	  return sort_order;                         // Found key is > range
         /* null -- exact match, go to next key part */
 	continue;
       }
       else if (field_is_null)
-	return -1;                              // NULL is less than any value
+	return -sort_order;                     // NULL is less than any value
       key++;					// Skip null byte
       store_length--;
     }
     if ((cmp=key_part->field->key_cmp(key, key_part->length)) < 0)
-      return -1;
+      return -sort_order;
     if (cmp > 0)
-      return 1;
+      return sort_order;
   }
   return 0;                                     // Keys are equal
 }
@@ -573,6 +574,9 @@ int key_rec_cmp(void *key_p, uchar *first_rec, uchar *second_rec)
     /* loop over every key part */
     do
     {
+      const int GREATER= key_part->key_part_flag & HA_REVERSE_SORT ? -1 : +1;
+      const int LESS= -GREATER;
+
       field= key_part->field;
 
       if (key_part->null_bit)
@@ -593,12 +597,12 @@ int key_rec_cmp(void *key_p, uchar *first_rec, uchar *second_rec)
             ; /* Fall through, no NULL fields */
           else
           {
-            DBUG_RETURN(+1);
+            DBUG_RETURN(GREATER);
           }
         }
         else if (!sec_is_null)
         {
-          DBUG_RETURN(-1);
+          DBUG_RETURN(LESS);
         }
         else
           goto next_loop; /* Both were NULL */
@@ -612,7 +616,7 @@ int key_rec_cmp(void *key_p, uchar *first_rec, uchar *second_rec)
       */
       if ((result= field->cmp_prefix(field->ptr+first_diff, field->ptr+sec_diff,
                                      key_part->length)))
-        DBUG_RETURN(result);
+        DBUG_RETURN(result * GREATER);
 next_loop:
       key_part++;
       key_part_num++;
