@@ -27,9 +27,8 @@ Query optimization produces two data structures:
 produce output of SHOW EXPLAIN, EXPLAIN [FORMAT=JSON], or 
 ANALYZE [FORMAT=JSON], without accessing the execution data structures.
 
-(the only exception is that Explain data structures keep Item* pointers,
-and we require that one might call item->print(QT_EXPLAIN) when printing
-FORMAT=JSON output)
+The exception is that Explain data structures have Item* pointers. See
+ExplainDataStructureLifetime below for details.
 
 === ANALYZE data ===
 EXPLAIN data structures have embedded ANALYZE data structures. These are 
@@ -74,7 +73,6 @@ class Json_writer;
 *************************************************************************************/
 
 
-const int FAKE_SELECT_LEX_ID= (int)UINT_MAX;
 
 class Explain_query;
 
@@ -108,7 +106,7 @@ public:
   };
 
   virtual enum explain_node_type get_type()= 0;
-  virtual int get_select_id()= 0;
+  virtual uint get_select_id()= 0;
 
   /**
     expression cache statistics
@@ -121,11 +119,13 @@ public:
   */
   enum explain_connection_type connection_type;
 
+protected:
   /* 
     A node may have children nodes. When a node's explain structure is 
     created, children nodes may not yet have QPFs. This is why we store ids.
   */
   Dynamic_array<int> children;
+public:
   void add_child(int select_no)
   {
     children.append(select_no);
@@ -134,12 +134,13 @@ public:
   virtual int print_explain(Explain_query *query, select_result_sink *output, 
                             uint8 explain_flags, bool is_analyze)=0;
   virtual void print_explain_json(Explain_query *query, Json_writer *writer, 
-                                  bool is_analyze)= 0;
+                                  bool is_analyze, bool no_tmp_tbl)= 0;
 
   int print_explain_for_children(Explain_query *query, select_result_sink *output, 
                                  uint8 explain_flags, bool is_analyze);
   void print_explain_json_for_children(Explain_query *query,
-                                       Json_writer *writer, bool is_analyze);
+                                       Json_writer *writer, bool is_analyze,
+                                       bool no_tmp_tbl);
   bool print_explain_json_cache(Json_writer *writer, bool is_analyze);
   virtual ~Explain_node(){}
 };
@@ -166,17 +167,17 @@ public:
 
   bool add_table(Explain_table_access *tab, Explain_query *query);
 
-  int get_select_id() { return select_id; }
+  uint get_select_id() { return select_id; }
 
-  int select_id;
+  uint select_id;
 
   int print_explain(Explain_query *query, select_result_sink *output,
                     uint8 explain_flags, bool is_analyze);
   void print_explain_json(Explain_query *query, Json_writer *writer, 
-                          bool is_analyze);
+                          bool is_analyze, bool no_tmp_tbl);
 
   void print_explain_json_interns(Explain_query *query, Json_writer *writer,
-                                  bool is_analyze);
+                                  bool is_analyze, bool no_tmp_tbl);
 
   /* A flat array of Explain structs for tables. */
   Explain_table_access** join_tabs;
@@ -260,7 +261,7 @@ public:
   int print_explain(Explain_query *query, select_result_sink *output, 
                     uint8 explain_flags, bool is_analyze);
   void print_explain_json(Explain_query *query, Json_writer *writer, 
-                          bool is_analyze);
+                          bool is_analyze, bool no_tmp_tbl);
   
   Table_access_tracker *get_using_temporary_read_tracker()
   {
@@ -303,7 +304,8 @@ public:
   Explain_aggr_filesort(MEM_ROOT *mem_root, bool is_analyze, 
                         Filesort *filesort);
 
-  void print_json_members(Json_writer *writer, bool is_analyze);
+  void print_json_members(Json_writer *writer, bool is_analyze,
+                          bool no_tmp_tbl);
 };
 
 class Explain_aggr_tmp_table : public Explain_aggr_node
@@ -324,16 +326,19 @@ class Explain_aggr_window_funcs : public Explain_aggr_node
 public:
   enum_explain_aggr_node_type get_type() { return AGGR_OP_WINDOW_FUNCS; }
 
-  void print_json_members(Json_writer *writer, bool is_analyze);
+  void print_json_members(Json_writer *writer, bool is_analyze,
+                          bool no_tmp_tbl);
   friend class Window_funcs_computation;
 };
 
 /////////////////////////////////////////////////////////////////////////////
 
 extern const char *unit_operation_text[4];
+extern const char *pushed_derived_text;
+extern const char *pushed_select_text;
 
 /*
-  Explain structure for a UNION.
+  Explain structure for a UNION [ALL].
 
   A UNION may or may not have "Using filesort".
 */
@@ -342,7 +347,7 @@ class Explain_union : public Explain_node
 {
 public:
   Explain_union(MEM_ROOT *root, bool is_analyze) : 
-    Explain_node(root),
+    Explain_node(root), union_members(PSI_INSTRUMENT_MEM),
     is_recursive_cte(false),
     fake_select_lex_explain(root, is_analyze)
   {}
@@ -350,7 +355,7 @@ public:
   enum explain_node_type get_type() { return EXPLAIN_UNION; }
   unit_common_op operation;
 
-  int get_select_id()
+  uint get_select_id()
   {
     DBUG_ASSERT(union_members.elements() > 0);
     return union_members.at(0);
@@ -375,7 +380,7 @@ public:
   int print_explain(Explain_query *query, select_result_sink *output, 
                     uint8 explain_flags, bool is_analyze);
   void print_explain_json(Explain_query *query, Json_writer *writer, 
-                          bool is_analyze);
+                          bool is_analyze, bool no_tmp_tbl);
 
   const char *fake_select_type;
   bool using_filesort;
@@ -414,36 +419,54 @@ class Explain_insert;
 /*
   Explain structure for a query (i.e. a statement).
 
-  This should be able to survive when the query plan was deleted. Currently, 
-  we do not intend for it survive until after query's MEM_ROOT is freed. It
-  does surivive freeing of query's items.
-   
-  For reference, the process of post-query cleanup is as follows:
+  This should be able to survive when the query plan was deleted. Currently,
+  we do not intend for it survive until after query's MEM_ROOT is freed.
+
+  == ExplainDataStructureLifetime ==
 
     >dispatch_command
     | >mysql_parse
-    | |  ...
-    | | lex_end()
-    | |  ...
-    | | >THD::cleanup_after_query
-    | | | ...
-    | | | free_items()
-    | | | ...
-    | | <THD::cleanup_after_query
+    | | ...
+    | |
+    | | explain->query_plan_ready(); // (1)
+    | |
+    | |   some_join->cleanup(); //  (2)
+    | |
+    | | explain->notify_tables_are_closed(); // (3)
+    | | close_thread_tables();  // (4)
+    | | ...
+    | | free_items(); // (5)
+    | | ...
     | |
     | <mysql_parse
     |
-    | log_slow_statement()
-    | 
+    | log_slow_statement() // (6)
+    |
     | free_root()
-    | 
+    |
     >dispatch_command
-  
-  That is, the order of actions is:
-    - free query's Items
-    - write to slow query log 
-    - free query's MEM_ROOT
-    
+
+  (1) - Query plan construction is finished and it is available for reading.
+
+  (2) - Temporary tables are freed. After this point,
+        we need to pass QT_DONT_ACCESS_TMP_TABLES to item->print(). Since
+        we don't track when #2 happens for each temp.table, we pass this
+        flag whenever we're printing the query plan for a SHOW command.
+        Also, we pass it when printing ANALYZE (?)
+
+  (3) - Notification about (4).
+  (4) - Tables used by the query are closed. One known consequence of this is
+        that the values of the const tables' fields are not available anymore.
+        We could use the same approach as in QT_DONT_ACCESS_TMP_TABLES to work
+        around that, but instead we disallow producing FORMAT=JSON output at
+        step #3. We also processing of SHOW command. The rationale is that
+        query is close to finish anyway.
+
+  (5) - Item objects are freed. After this, it's certainly not possible to
+        print them into FORMAT=JSON output.
+
+  (6) - We may decide to log tabular EXPLAIN output to the slow query log.
+
 */
 
 class Explain_query : public Sql_alloc
@@ -475,17 +498,24 @@ public:
   /* Return tabular EXPLAIN output as a text string */
   bool print_explain_str(THD *thd, String *out_str, bool is_analyze);
 
-  void print_explain_json(select_result_sink *output, bool is_analyze);
+  int print_explain_json(select_result_sink *output, bool is_analyze,
+                         bool is_show_cmd,
+                         ulonglong query_time_in_progress_ms= 0);
 
   /* If true, at least part of EXPLAIN can be printed */
   bool have_query_plan() { return insert_plan || upd_del_plan|| get_node(1) != NULL; }
 
   void query_plan_ready();
+  void notify_tables_are_closed();
 
   MEM_ROOT *mem_root;
 
   Explain_update *get_upd_del_plan() { return upd_del_plan; }
 private:
+  bool print_query_blocks_json(Json_writer *writer, const bool is_analyze, const bool is_show_cmd);
+  void print_query_optimization_json(Json_writer *writer);
+  void send_explain_json_to_output(Json_writer *writer, select_result_sink *output);
+ 
   /* Explain_delete inherits from Explain_update */
   Explain_update *upd_del_plan;
 
@@ -495,7 +525,7 @@ private:
   Dynamic_array<Explain_union*> unions;
   Dynamic_array<Explain_select*> selects;
   
-  THD *thd; // for APC start/stop
+  THD *stmt_thd; // for APC start/stop
   bool apc_enabled;
   /* 
     Debugging aid: count how many times add_node() was called. Ideally, it
@@ -504,6 +534,11 @@ private:
     is unacceptable.
   */
   longlong operations;
+#ifndef DBUG_OFF
+  bool can_print_json= false;
+#endif
+
+  Exec_time_tracker optimization_time_tracker;
 };
 
 
@@ -548,6 +583,7 @@ enum explain_extra_tag
   ET_CONST_ROW_NOT_FOUND,
   ET_UNIQUE_ROW_NOT_FOUND,
   ET_IMPOSSIBLE_ON_CONDITION,
+  ET_TABLE_FUNCTION,
 
   ET_total
 };
@@ -586,6 +622,8 @@ class Explain_index_use : public Sql_alloc
 {
   char *key_name;
   uint key_len;
+  char *filter_name;
+  uint filter_len;
 public:
   String_list key_parts_list;
   
@@ -598,12 +636,46 @@ public:
   {
     key_name= NULL;
     key_len= (uint)-1;
+    filter_name= NULL;
+    filter_len= (uint)-1;
   }
   bool set(MEM_ROOT *root, KEY *key_name, uint key_len_arg);
   bool set_pseudo_key(MEM_ROOT *root, const char *key_name);
 
   inline const char *get_key_name() const { return key_name; }
   inline uint get_key_len() const { return key_len; }
+  //inline const char *get_filter_name() const { return filter_name; }
+};
+
+
+/*
+  Query Plan data structure for Rowid filter.
+*/
+class Explain_rowid_filter : public Sql_alloc
+{
+public:
+  /* Quick select used to collect the rowids into filter */
+  Explain_quick_select *quick;
+
+  /* How many rows the above quick select is expected to return */
+  ha_rows rows;
+
+  /* Expected selectivity for the filter */
+  double selectivity;
+
+  /* Tracker with the information about how rowid filter is executed */
+  Rowid_filter_tracker *tracker;
+
+  void print_explain_json(Explain_query *query, Json_writer *writer,
+                          bool is_analyze);
+
+  /*
+    TODO:
+      Here should be ANALYZE members:
+      - r_rows for the quick select
+      - An object that tracked the table access time
+      - real selectivity of the filter.
+  */
 };
 
 
@@ -673,6 +745,7 @@ public:
   void print_json(Json_writer *writer, bool is_analyze);
 };
 
+
 /*
   EXPLAIN data structure for a single JOIN_TAB.
 */
@@ -692,7 +765,8 @@ public:
     cache_cond(NULL),
     pushed_index_cond(NULL),
     sjm_nest(NULL),
-    pre_join_sort(NULL)
+    pre_join_sort(NULL),
+    rowid_filter(NULL)
   {}
   ~Explain_table_access() { delete sjm_nest; }
 
@@ -797,21 +871,26 @@ public:
   /* Tracker for reading the table */
   Table_access_tracker tracker;
   Exec_time_tracker op_tracker;
+  Gap_time_tracker extra_time_tracker;
+
   Table_access_tracker jbuf_tracker;
   
+  Explain_rowid_filter *rowid_filter;
+
   int print_explain(select_result_sink *output, uint8 explain_flags, 
                     bool is_analyze,
                     uint select_id, const char *select_type,
                     bool using_temporary, bool using_filesort);
   void print_explain_json(Explain_query *query, Json_writer *writer,
-                          bool is_analyze);
+                          bool is_analyze, bool no_tmp_tbl);
 
 private:
   void append_tag_name(String *str, enum explain_extra_tag tag);
   void fill_key_str(String *key_str, bool is_json) const;
-  void fill_key_len_str(String *key_len_str) const;
+  void fill_key_len_str(String *key_len_str, bool is_json) const;
   double get_r_filtered();
-  void tag_to_json(Json_writer *writer, enum explain_extra_tag tag);
+  void tag_to_json(Json_writer *writer, enum explain_extra_tag tag,
+                   bool no_tmp_tbl);
 };
 
 
@@ -835,7 +914,7 @@ public:
   {}
 
   virtual enum explain_node_type get_type() { return EXPLAIN_UPDATE; }
-  virtual int get_select_id() { return 1; /* always root */ }
+  virtual uint get_select_id() { return 1; /* always root */ }
 
   const char *select_type;
 
@@ -894,7 +973,7 @@ public:
   virtual int print_explain(Explain_query *query, select_result_sink *output, 
                             uint8 explain_flags, bool is_analyze);
   virtual void print_explain_json(Explain_query *query, Json_writer *writer,
-                                  bool is_analyze);
+                                  bool is_analyze, bool no_tmp_tbl);
 };
 
 
@@ -915,12 +994,12 @@ public:
   StringBuffer<64> table_name;
 
   enum explain_node_type get_type() { return EXPLAIN_INSERT; }
-  int get_select_id() { return 1; /* always root */ }
+  uint get_select_id() { return 1; /* always root */ }
 
   int print_explain(Explain_query *query, select_result_sink *output, 
                     uint8 explain_flags, bool is_analyze);
   void print_explain_json(Explain_query *query, Json_writer *writer, 
-                          bool is_analyze);
+                          bool is_analyze, bool no_tmp_tbl);
 };
 
 
@@ -942,12 +1021,12 @@ public:
   bool deleting_all_rows;
 
   virtual enum explain_node_type get_type() { return EXPLAIN_DELETE; }
-  virtual int get_select_id() { return 1; /* always root */ }
+  virtual uint get_select_id() { return 1; /* always root */ }
 
   virtual int print_explain(Explain_query *query, select_result_sink *output, 
                             uint8 explain_flags, bool is_analyze);
   virtual void print_explain_json(Explain_query *query, Json_writer *writer,
-                                  bool is_analyze);
+                                  bool is_analyze, bool no_tmp_tbl);
 };
 
 
