@@ -22,6 +22,10 @@
 
 #include "strings_def.h"
 #include <m_ctype.h>
+#include "ctype-simple.h"
+
+const char charset_name_binary[]= "binary";
+#define charset_name_binary_length (sizeof(charset_name_binary)-1)
 
 static const uchar ctype_bin[]=
 {
@@ -125,6 +129,17 @@ static int my_strnncollsp_binary(CHARSET_INFO * cs __attribute__((unused)),
 }
 
 
+static int my_strnncollsp_nchars_binary(CHARSET_INFO * cs __attribute__((unused)),
+                                        const uchar *s, size_t slen,
+                                        const uchar *t, size_t tlen,
+                                        size_t nchars)
+{
+  set_if_smaller(slen, nchars);
+  set_if_smaller(tlen, nchars);
+  return my_strnncoll_binary(cs, s, slen, t, tlen, 0);
+}
+
+
 static int my_strnncoll_8bit_bin(CHARSET_INFO * cs __attribute__((unused)),
                                  const uchar *s, size_t slen,
                                  const uchar *t, size_t tlen,
@@ -199,6 +214,17 @@ static int my_strnncollsp_8bit_bin(CHARSET_INFO * cs __attribute__((unused)),
 }
 
 
+static int my_strnncollsp_nchars_8bit_bin(CHARSET_INFO * cs,
+                                          const uchar *a, size_t a_length,
+                                          const uchar *b, size_t b_length,
+                                          size_t nchars)
+{
+  set_if_smaller(a_length, nchars);
+  set_if_smaller(b_length, nchars);
+  return my_strnncollsp_8bit_bin(cs, a, a_length, b, b_length);
+}
+
+
 static int my_strnncollsp_8bit_nopad_bin(CHARSET_INFO * cs
                                          __attribute__((unused)),
                                          const uchar *a, size_t a_length,
@@ -487,6 +513,7 @@ MY_COLLATION_HANDLER my_collation_8bit_bin_handler =
   my_coll_init_8bit_bin,
   my_strnncoll_8bit_bin,
   my_strnncollsp_8bit_bin,
+  my_strnncollsp_nchars_8bit_bin,
   my_strnxfrm_8bit_bin,
   my_strnxfrmlen_simple,
   my_like_range_simple,
@@ -494,7 +521,11 @@ MY_COLLATION_HANDLER my_collation_8bit_bin_handler =
   my_strcasecmp_bin,
   my_instr_bin,
   my_hash_sort_8bit_bin,
-  my_propagate_simple
+  my_propagate_simple,
+  my_min_str_8bit_simple,
+  my_max_str_8bit_simple,
+  my_ci_get_id_generic,
+  my_ci_get_collation_name_generic
 };
 
 
@@ -503,6 +534,7 @@ MY_COLLATION_HANDLER my_collation_8bit_nopad_bin_handler =
   my_coll_init_8bit_bin,
   my_strnncoll_8bit_bin,
   my_strnncollsp_8bit_nopad_bin,
+  my_strnncollsp_nchars_8bit_bin,
   my_strnxfrm_8bit_nopad_bin,
   my_strnxfrmlen_simple,
   my_like_range_simple,
@@ -510,7 +542,11 @@ MY_COLLATION_HANDLER my_collation_8bit_nopad_bin_handler =
   my_strcasecmp_bin,
   my_instr_bin,
   my_hash_sort_bin,
-  my_propagate_simple
+  my_propagate_simple,
+  my_min_str_8bit_simple_nopad,
+  my_max_str_8bit_simple,
+  my_ci_get_id_generic,
+  my_ci_get_collation_name_generic
 };
 
 
@@ -519,6 +555,7 @@ static MY_COLLATION_HANDLER my_collation_binary_handler =
   NULL,			/* init */
   my_strnncoll_binary,
   my_strnncollsp_binary,
+  my_strnncollsp_nchars_binary,
   my_strnxfrm_8bit_bin,
   my_strnxfrmlen_simple,
   my_like_range_simple,
@@ -526,7 +563,11 @@ static MY_COLLATION_HANDLER my_collation_binary_handler =
   my_strcasecmp_bin,
   my_instr_bin,
   my_hash_sort_bin,
-  my_propagate_simple
+  my_propagate_simple,
+  my_min_str_8bit_simple_nopad,
+  my_max_str_8bit_simple,
+  my_ci_get_id_generic,
+  my_ci_get_collation_name_generic
 };
 
 
@@ -560,6 +601,7 @@ static MY_CHARSET_HANDLER my_charset_handler=
   my_well_formed_char_length_8bit,
   my_copy_8bit,
   my_wc_mb_bin,
+  my_wc_to_printable_generic
 };
 
 
@@ -567,8 +609,8 @@ struct charset_info_st my_charset_bin =
 {
     63,0,0,			/* number        */
     MY_CS_COMPILED|MY_CS_BINSORT|MY_CS_PRIMARY|MY_CS_NOPAD,/* state */
-    "binary",			/* cs name    */
-    "binary",			/* name          */
+    { charset_name_binary, charset_name_binary_length},	/* cs name    */
+    { STRING_WITH_LEN("binary")},	                /* name          */
     "",				/* comment       */
     NULL,			/* tailoring     */
     ctype_bin,			/* ctype         */
@@ -590,7 +632,11 @@ struct charset_info_st my_charset_bin =
     255,			/* max_sort_char */
     0,                          /* pad char      */
     0,                          /* escape_with_backslash_is_dangerous */
-    1,                          /* levels_for_order   */
+    MY_CS_COLL_LEVELS_S1,
     &my_charset_handler,
     &my_collation_binary_handler
 };
+
+
+struct charset_info_st my_collation_contextually_typed_binary= {0};
+struct charset_info_st my_collation_contextually_typed_default= {0};
