@@ -35,12 +35,14 @@
 
 #include "strings_def.h"
 #include <m_ctype.h>
+#include "ctype-simple.h"
 #include "t_ctype.h"
 #include <my_sys.h>
 
 #ifdef HAVE_CHARSET_tis620
 
 const char charset_name_tis620[]= "tis620";
+#define charset_name_tis620_length  sizeof(charset_name_tis620)-1
 
 #define M  L_MIDDLE
 #define U  L_UPPER
@@ -862,7 +864,11 @@ static MY_COLLATION_HANDLER my_collation_ci_handler =
     my_strcasecmp_8bit,
     my_instr_simple,				/* QQ: To be fixed */
     my_hash_sort_simple,
-    my_propagate_simple
+    my_propagate_simple,
+    my_min_str_8bit_simple,
+    my_max_str_8bit_simple,
+    my_ci_get_id_generic,
+    my_ci_get_collation_name_generic
 };
 
 static MY_COLLATION_HANDLER my_collation_nopad_ci_handler =
@@ -878,7 +884,11 @@ static MY_COLLATION_HANDLER my_collation_nopad_ci_handler =
     my_strcasecmp_8bit,
     my_instr_simple,				/* QQ: To be fixed */
     my_hash_sort_simple_nopad,
-    my_propagate_simple
+    my_propagate_simple,
+    my_min_str_8bit_simple_nopad,
+    my_max_str_8bit_simple,
+    my_ci_get_id_generic,
+    my_ci_get_collation_name_generic
 };
 
 static MY_CHARSET_HANDLER my_charset_handler=
@@ -920,8 +930,8 @@ struct charset_info_st my_charset_tis620_thai_ci=
 {
     18,0,0,		/* number    */
     MY_CS_COMPILED|MY_CS_PRIMARY|MY_CS_STRNXFRM|MY_CS_NON1TO1, /* state     */
-    charset_name_tis620,		/* cs name    */
-    "tis620_thai_ci",	/* name      */
+    { charset_name_tis620, charset_name_tis620_length },        /* cs name    */
+    { STRING_WITH_LEN("tis620_thai_ci") },	               /* name      */
     "",			/* comment   */
     NULL,		/* tailoring */
     ctype_tis620,
@@ -943,7 +953,7 @@ struct charset_info_st my_charset_tis620_thai_ci=
     255,		/* max_sort_char */
     ' ',                /* pad char      */
     0,                  /* escape_with_backslash_is_dangerous */
-    1,                  /* levels_for_order   */
+    MY_CS_COLL_LEVELS_S1,
     &my_charset_handler,
     &my_collation_ci_handler
 };
@@ -952,8 +962,8 @@ struct charset_info_st my_charset_tis620_bin=
 {
     89,0,0,		/* number    */
     MY_CS_COMPILED|MY_CS_BINSORT,	/* state     */
-    charset_name_tis620,		/* cs name    */
-    "tis620_bin",	/* name      */
+    { charset_name_tis620, charset_name_tis620_length },	/* cs name    */
+    { STRING_WITH_LEN("tis620_bin") },	                        /* name      */
     "",			/* comment   */
     NULL,		/* tailoring */
     ctype_tis620,
@@ -975,7 +985,7 @@ struct charset_info_st my_charset_tis620_bin=
     255,		/* max_sort_char */
     ' ',                /* pad char      */
     0,                  /* escape_with_backslash_is_dangerous */
-    1,                  /* levels_for_order   */
+    MY_CS_COLL_LEVELS_S1,
     &my_charset_handler,
     &my_collation_8bit_bin_handler
 };
@@ -985,8 +995,8 @@ struct charset_info_st my_charset_tis620_thai_nopad_ci=
 {
     MY_NOPAD_ID(18),0,0,   /* number           */
     MY_CS_COMPILED|MY_CS_STRNXFRM|MY_CS_NON1TO1|MY_CS_NOPAD, /* state */
-    charset_name_tis620,              /* cs name          */
-    "tis620_thai_nopad_ci",/* name             */
+    { charset_name_tis620, charset_name_tis620_length },	/* cs name    */
+    { STRING_WITH_LEN("tis620_thai_nopad_ci") },                /* name        */
     "",                    /* comment          */
     NULL,                  /* tailoring        */
     ctype_tis620,
@@ -1008,7 +1018,7 @@ struct charset_info_st my_charset_tis620_thai_nopad_ci=
     255,                   /* max_sort_char    */
     ' ',                   /* pad char         */
     0,                     /* escape_with_backslash_is_dangerous */
-    1,                     /* levels_for_order */
+    MY_CS_COLL_LEVELS_S1,
     &my_charset_handler,
     &my_collation_nopad_ci_handler
 };
@@ -1018,8 +1028,8 @@ struct charset_info_st my_charset_tis620_nopad_bin=
 {
     MY_NOPAD_ID(89),0,0,   /* number           */
     MY_CS_COMPILED|MY_CS_BINSORT|MY_CS_NOPAD, /* state */
-    charset_name_tis620,              /* cs name          */
-    "tis620_nopad_bin",    /* name             */
+    { charset_name_tis620, charset_name_tis620_length },	/* cs name    */
+    { STRING_WITH_LEN("tis620_nopad_bin") },                    /* name      */
     "",                    /* comment          */
     NULL,                  /* tailoring        */
     ctype_tis620,
@@ -1041,7 +1051,7 @@ struct charset_info_st my_charset_tis620_nopad_bin=
     255,                   /* max_sort_char    */
     ' ',                   /* pad char         */
     0,                     /* escape_with_backslash_is_dangerous */
-    1,                     /* levels_for_order */
+    MY_CS_COLL_LEVELS_S1,
     &my_charset_handler,
     &my_collation_8bit_nopad_bin_handler
 };
